@@ -13,6 +13,7 @@ newtype PString = PatString { unPS    :: String } deriving Show
 newtype Path    = Path      { unP     :: String } deriving Show
 newtype COpts   = COpts     { unCOpts :: CompOptions } deriving Show
 
+alpha0, alpha :: String
 alpha0 = extSeparator : "-^!" ++ ['a'..'z'] ++ ['0'..'9']
 alpha  = pathSeparators ++ alpha0
 
@@ -31,20 +32,25 @@ instance Arbitrary PString where
       s <- mapM (const $ frequency xs) [1..size]
       return.PatString $ concat s
 
+   shrink (PatString s) = map PatString (shrink s)
+
 instance Arbitrary Path where
    arbitrary = sized $ \size -> do
       s <- mapM (const $ plain alpha) [1..size `mod` 16]
       return.Path $ concat s
 
+   shrink (Path s) = map Path (shrink s)
+
 instance Arbitrary COpts where
    arbitrary = do
       [a,b,c,d,e,f] <- vector 6
       return.COpts $ CompOptions a b c d e f False
 
-plain from = sized $ \size -> do
-   s <- mapM (const $ elements from) [0..size `mod` 3]
-   return s
 
+plain :: String -> Gen String
+plain from = sized $ \size -> mapM (const $ elements from) [0..size `mod` 3]
+
+charRange :: Gen String
 charRange = do
    s <- plain alpha0
    if s `elem` ["^","!"]
@@ -54,6 +60,7 @@ charRange = do
       else
          return$ "[" ++ s ++       "]"
 
+openRange :: Gen String
 openRange = do
    probA <- choose (0,1) :: Gen Float
    probB <- choose (0,1) :: Gen Float
@@ -71,10 +78,14 @@ openRange = do
       , ">"
       ]
 
+-- Not in Data.Either until base-4.7 (GHC 7.8)
+isRight :: Either a b -> Bool
+isRight (Right _) = True
+isRight _ = False
+
+fromRight :: Either a b -> b
 fromRight (Right x) = x
 fromRight _         = error "fromRight :: Left"
 
-isRight (Right _) = True
-isRight _         = False
-
+(-->) :: Bool -> Bool -> Bool
 a --> b = not a || b
